/*** ESSENTIAL STYLES ***/
.sf-menu, .sf-menu * {
	margin: 0;
	padding: 0;
	list-style: none;
}
.sf-menu li {
	position: relative;
}
.sf-menu ul {
	position: absolute;
	display: none;
	top: 100%;
	right: 0;
	z-index: 99;
}
.sf-menu > li {
	float: left;
}
.sf-menu li:hover > ul,
.sf-menu li.sfHover > ul {
	display: block;
}
.sf-menu > li.right_menu{
    float:right;
}

.sf-menu a {
	display: block;
	position: relative;
}
.sf-menu ul ul {
	top: 0;
  left:0;
}


/*** DEMO SKIN ***/
.sf-menu {
	float: left;
	margin-bottom: 1em;
}
.sf-menu ul {
	box-shadow: 2px 2px 6px rgba(0,0,0,.2);
	min-width: 16em; /* allow long menu items to determine submenu width */
	*width: 16em; /* no auto sub width for IE7, see white-space comment below */
  border: 1px solid #C2C2C2;
  background: #E4E4E4;
  border-left:none;
  padding-bottom:2px;
}
.sf-menu li a {
	display:block;
  height: 20px;
  color: #074A86;
  border-right: 1px solid #c2c2c2;
  font-weight:bold;
  font-size:15px;
  text-decoration:none;
  padding:7px 20px;
	zoom: 1; /* IE7 */
}
.sf-menu li {
	white-space: nowrap; /* no need for Supersubs plugin */
	*white-space: normal; /* ...unless you support IE7 (let it wrap) */
}
.sf-menu li.right_menu > a{
  border-left: 1px solid #c2c2c2;
  border-right:none;
  width: 35px;
  padding:7px;
}
.sf-menu li.main_menu{
  background: url(../images/main_menu.png) center no-repeat;
}
.sf-menu li.slapos_run{
  background: url(../images/run_button.png) center no-repeat;
}
.sf-menu li.slapos_stop{
  background: url(../images/stop_button.png) center no-repeat;
}
.sf-menu li:hover,
.sf-menu li.sfHover {
  color: #fff;/*#0271BF;*/
  background:#c2c2c2;
	/* only transition out, not in */
	-webkit-transition: none;
	transition: none;
}
.sf-menu li.main_menu:hover,
.sf-menu li.main_menu.sfHover {
  background: #c2c2c2 url(../images/main_menu_hover.png) center no-repeat;
}
.sf-menu li.slapos_stop:hover,
.sf-menu li.slapos_stop.sfHover{
  background: #c2c2c2 url(../images/stop_button2.png) center no-repeat;
}
.sf-menu li.slapos_run:hover,
.sf-menu li.slapos_run.sfHover{
  background: #c2c2c2 url(../images/run_button2.png) center no-repeat;
}
.sf-menu li a:hover{
  color: #fff;
}

.sf-menu ul li {
	background: #E4E4E4;
  cursor: pointer;
  color: #000;
}
.sf-menu ul li.sep{border-bottom: 1px solid #c2c2c2; margin: 3px 0;}
.sf-menu ul li.sep:hover{
  background: #E4E4E4;
}
.sf-menu ul ul li {
	background: #E4E4E4;
}
.sf-menu ul ul{
  border: 1px solid #C2C2C2;
  border-left:none;
}
.sf-menu ul li a{
  font-weight:normal;
  font-size:14px;
  color: #000;
  height: 16px;
  padding:7px;
  padding-left: 15px;
  border:none;
}
.sf-menu ul li:hover,
.sf-menu ul li.sfHoverhover{
  background: #A3A3A3;
}
.sf-menu ul li:hover a,
.sf-menu ul li.sfHoverhover a{
  color: #fff;
}

/*** arrows (for all except IE7) **/
.sf-arrows .sf-with-ul {
	padding-right: 2.5em;
	*padding-right: 1em; /* no CSS arrows for IE7 (lack pseudo-elements) */
}
/* styling for both css and generated arrows */
.sf-arrows .sf-with-ul:after {
	content: '';
	position: absolute;
	top: 50%;
	right: 1em;
	margin-top: -3px;
	height: 0;
	width: 0;
	/* order of following 3 rules important for fallbacks to work */
	border: 5px solid transparent;
	border-top-color: #dFeEFF; /* edit this to suit design (no rgba in IE8) */
	border-top-color: rgba(255,255,255,.5);
}
.sf-arrows > li > .sf-with-ul:focus:after,
.sf-arrows > li:hover > .sf-with-ul:after,
.sf-arrows > .sfHover > .sf-with-ul:after {
	border-top-color: white; /* IE8 fallback colour */
}
/* styling for right-facing arrows */
.sf-arrows ul .sf-with-ul:after {
	margin-top: -5px;
	margin-right: -3px;
	border-color: transparent;
	border-left-color: #dFeEFF; /* edit this to suit design (no rgba in IE8) */
	border-left-color: rgba(255,255,255,.5);
}
.sf-arrows ul li > .sf-with-ul:focus:after,
.sf-arrows ul li:hover > .sf-with-ul:after,
.sf-arrows ul .sfHover > .sf-with-ul:after {
	border-left-color: white;
}
